/*
 * TranslationUnitStore.cpp
 *
 *  Created on: 10.01.2013
 *      Author: kaeshammer
 */

#include "TranslationUnitStore.h"

#include <sstream>
#include <iterator>
#include <iostream>
#include <algorithm>

#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>


namespace Moses {
namespace LCFRS {

void TranslationUnit::addToSource(int i)
{
  m_sourceInd.insert(i);
}

void TranslationUnit::addToTarget(int i)
{
  m_targetInd.insert(i);
}

const std::set<int>& TranslationUnit::getSourceIndeces() const
{
  return m_sourceInd;
}

const std::set<int>& TranslationUnit::getTargetIndeces() const
{
  return m_targetInd;
}

TranslationUnit::TranslationUnit()
{

}

TranslationUnit::~TranslationUnit()
{

}

std::string TranslationUnit::toString() const
{
  std::stringstream result;
  std::copy(m_sourceInd.begin(), m_sourceInd.end(), std::ostream_iterator<int>(result, " "));

  result << "--- ";
  std::copy(m_targetInd.begin(), m_targetInd.end(), std::ostream_iterator<int>(result, " "));
  return result.str();
}

std::string TranslationUnit::toStringTU() const
{
  std::string res;
  for (std::set<int>::iterator iter = m_sourceInd.begin();
      iter != m_sourceInd.end(); iter++) {
    res.append(boost::lexical_cast<std::string>(*iter + 1));
    res.append(" ");
  }
  boost::trim(res);
  res.append(":");
  for (std::set<int>::iterator iter = m_targetInd.begin();
      iter != m_targetInd.end(); iter++) {
    res.append(boost::lexical_cast<std::string>(*iter + 1));
    res.append(" ");
  }
  boost::trim(res);
  return res;
}

std::string TranslationUnit::toStringTUWithoutUnaligned(IntMap& in2outSource, IntMap& in2outTarget) const
{
	std::string res;
	for (std::set<int>::iterator iter = m_sourceInd.begin();
	    iter != m_sourceInd.end(); iter++) {
      int i = in2outSource[*iter]+1;
	  res.append(boost::lexical_cast<std::string>(i));
	  res.append(" ");
	}
	boost::trim(res);
	res.append(":");
	for (std::set<int>::iterator iter = m_targetInd.begin();
	    iter != m_targetInd.end(); iter++) {
	  int j = in2outTarget[*iter]+1;
	  res.append(boost::lexical_cast<std::string>(j));
	  res.append(" ");
	}
	boost::trim(res);
	return res;
}


bool TranslationUnit::operator<(const TranslationUnit& other) const
{
  if (m_sourceInd.empty())
  {
    return true;
  }
  if (other.getSourceIndeces().empty())
  {
    return false;
  }
  const int smallest = *(std::min_element( m_sourceInd.begin(), m_sourceInd.end() ) );
  const int otherSmallest = *(std::min_element( other.getSourceIndeces().begin(), other.getSourceIndeces().end() ));
  return smallest < otherSmallest;
}

TranslationUnitStore::TranslationUnitStore(Moses::LCFRS::Alignment& alignment,
    std::vector<std::string> sourceTokens, std::vector<std::string> targetTokens)
{
  int countS = sourceTokens.size();
  int countT = targetTokens.size();
  numSource = countS;
  numTarget = countT;
  std::vector<bool> sourceTreated;
  std::vector<bool> targetTreated;
  for (int i = 0; i < countS; i++) {
    sourceTreated.push_back(false);
  }
  for (int i = 0; i < countT; i++) {
    targetTreated.push_back(false);
  }

  for (int i = 0; i < countT; i++)
  {
    if (targetTreated[i])
    {
      continue;
    }

    TranslationUnit tu;
    //std::cout << "new TU" << std::endl;
    addToTU(alignment, tu, i, sourceTreated, targetTreated);
    m_tus.push_back(tu);
  }
  for (unsigned int i = 0; i < sourceTokens.size(); i++) {
    if (sourceTreated[i])
    {
      continue;
    }
    TranslationUnit tu;
    tu.addToSource(i);
    m_sourceTus.insert(std::pair<int,TranslationUnit>(i,tu));
    m_tus.push_back(tu);
  }
}

void TranslationUnitStore::addToTU(Moses::LCFRS::Alignment& a, TranslationUnit& tu,
    int i, std::vector<bool>& sourceTreated, std::vector<bool>& targetTreated) {
  targetTreated[i] = true;
  tu.addToTarget(i);
  m_targetTus.insert(std::pair<int,TranslationUnit>(i,tu));
  //std::cout << "target " << i << std::endl;

  for (Moses::LCFRS::Alignment::const_iterator p(a.begin()); p != a.end(); ++p) {
    if (p->second == i) {
      int j = p->first;
      if (sourceTreated[j]) {
        continue;
      }
      sourceTreated[j] = true;
      tu.addToSource(j);
      m_sourceTus.insert(std::pair<int,TranslationUnit>(j,tu));

      std::vector<int> otherTargets;
      for (Moses::LCFRS::Alignment::const_iterator p2(a.begin()); p2 != a.end(); ++p2) {
        if (p2->first == j && !targetTreated[p2->second]) {
          otherTargets.push_back(p2->second);
        }
      }

      for (unsigned int k = 0; k < otherTargets.size(); k++) {
        addToTU(a, tu, otherTargets[k], sourceTreated, targetTreated);
      }
    }
  }
}

std::string TranslationUnitStore::toString() const
{
  std::string s;
  for (unsigned int i = 0; i < m_tus.size(); i++)
  {
    s.append(m_tus[i].toString()+'\n');
  }
  s.append("\n");
  return s;
}

std::string TranslationUnitStore::toStringTU() const
{
  std::string s;
  for (unsigned int i = 0; i < m_tus.size(); i++)
  {
    if ((m_tus[i]).isOnlySource() || (m_tus[i]).isOnlyTarget()) {
      continue;
    }
    s.append(m_tus[i].toStringTU()+'\t');
  }
  boost::trim(s);
  s.append("\n");
  return s;
}

std::string TranslationUnitStore::toStringTUWithoutUnaligned(IntMap& in2outSource, IntMap& in2outTarget) const
{
  std::string s;
  for (unsigned int i = 0; i < m_tus.size(); i++)
  {
    if ((m_tus[i]).isOnlySource() || (m_tus[i]).isOnlyTarget()) {
      continue;
    }
    s.append(m_tus[i].toStringTUWithoutUnaligned(in2outSource, in2outTarget)+'\t');
  }
  boost::trim(s);
  s.append("\n");
  return s;
}



}
}
